/*
Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences, Science and Technology Facilities Council, and Xlab d.o.o.
All rights reserved.

This file is part of VOM. VOM is part of XtreemOS, a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details. The XtreemOS project has been developed with the financial support of the European Commission's IST program under contract #FP6-033576.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the XtreemOS project nor the names of its partners and/or contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
Ext.app.SearchField = Ext.extend(Ext.form.TwinTriggerField, {
    initComponent: function(){
        Ext.app.SearchField.superclass.initComponent.call(this);
        this.on('specialkey', function(f, e){
            if (e.getKey() == e.ENTER) {
                this.onTrigger2Click();
            }
        }, this);
    },
    
    validationEvent: false,
    validateOnBlur: false,
    trigger1Class: 'x-form-clear-trigger',
    trigger2Class: 'x-form-search-trigger',
    hideTrigger1: true,
    width: 180,
    hasSearch: false,
    paramName: 'query',
    
    onTrigger1Click: function(){
        if (this.hasSearch) {
            this.el.dom.value = '';
            var o = {
                start: 0
            };
            this.store.baseParams = this.store.baseParams ||
            {};
            this.store.baseParams[this.paramName] = '';
            this.store.reload({
                params: o
            });
            this.triggers[0].hide();
            this.hasSearch = false;
        }
    },
    
    onTrigger2Click: function(){
        var v = this.getRawValue();
        if (v.length < 1) {
            this.onTrigger1Click();
            return;
        }
        var o = {
            start: 0
        };
        this.store.baseParams = this.store.baseParams ||
        {};
        this.store.baseParams[this.paramName] = v;
        this.store.reload({
            params: o
        });
        this.hasSearch = true;
        this.triggers[0].show();
    }
});

var addNewRCA = function(store){
    var winId = "createNewRCA";
    
    var myWin = Ext.getCmp(winId);
    if (myWin != null) {
        myWin.show();
        return;
    }
    
    function checkSubmited(){
        var name = Ext.getCmp('voname');
        if (!name.el.dom.value == null || name.el.dom.value.length <= 0) {
            Ext.MessageBox.alert("Create A VO", "vo name is null");
            name.focus();
            return false;
        }
        
        return true;
    }
    
    var fs = new Ext.FormPanel({
        labelWidth: 70,
        bodyStyle: 'padding:15px',
        border: false,
        frame: true,
        waitMsgTarget: true,
        url: '../model/rca_util.jsp?func=add',
        layout: 'fit',
        items: [{
            layout: 'column',
            items: [{
                columnWidth: .9,
                layout: 'form',
                items: [{
                    xtype: 'textfield',
                    fieldLabel: 'RCA Name',
                    name: 'rcaname',
                    id: 'rcaname',
                    allowBlank: false,
                    anchor: '95%'
                }, {
                    xtype: 'textfield',
                    fieldLabel: 'Description',
                    name: 'desc',
                    id: 'desc',
                    anchor: '95%'
                }, {
                    xtype: 'textfield',
                    fieldLabel: 'DIXI Host',
                    name: 'host',
                    id: 'host',
                    allowBlank: false,
                    anchor: '95%'
                }, {
                    xtype: 'textfield',
                    fieldLabel: 'DIXI Port',
                    name: 'port',
                    id: 'port',
                    allowBlank: false,
                    anchor: '95%'
                }, {
                    xtype: 'textfield',
                    fieldLabel: 'Desc',
                    name: 'desc',
                    id: 'desc',
                    anchor: '95%'
                }]
            }]
        }],
        
        buttons: [{
            text: 'Create',
            handler: function(){
                var form = fs.getForm();
                var rcaname = Ext.getCmp('rcaname').el.dom.value;
                
                form.submit({
                    waitTitle: 'Connecting',
                    waitMsg: 'Waiting...',
                    success: function(form, action){
                        Ext.Msg.alert('Create a RCA', "Successfully create a RCA:!" + rcaname);
                        form.reset();
                        myWin.close();
                        if (store) 
                            store.load();
                    },
                    failure: function(form, action){
                        if (action.failureType == 'server') {
                            obj = Ext.util.JSON.decode(action.response.responseText);
                            Ext.Msg.alert('Server Failed!', obj.errors.reason);
                        }
                        else {
                            Ext.Msg.alert('Warning!', 'Server is unreachable:' + action.response.responseText);
                        }
                    }
                });
            }
        }, {
            text: 'Cancel',
            handler: function(){
                myWin.close();
            }
        }]
    });
    
    myWin = new Ext.Window({
        id: winId,
        width: 400,
        //height: 300,
        plain: true,
        title: 'RCA Creation',
        layout: 'form',
        border: false,
        closeAction: 'hide',
        closable: true,
        items: fs,
        rendTo: 'form-ct'
    });
    myWin.show(this);
    
};
Ext.onReady(function(){
    Ext.QuickTips.init();
    
    var store = new Ext.data.JsonStore({
        url: "../model/list_rcas.jsp",
        totalProperty: "results",
        root: "rows",
        id: "id",
        fields: ['id', 'host', 'port', 'desc', 'name', 'owner']
    });
    var grid = new Ext.grid.GridPanel({
        store: store,
        loadMask: true,
        id: 'rcagrid',
        applyTo: 'form-ct',
        title: 'Register RCAs',
        columns: [new Ext.grid.CheckboxSelectionModel(), {
            header: "Id",
            width: 30,
            sortable: true,
            dataIndex: 'id'
        }, {
            header: "Name",
            width: 120,
            sortable: true,
            dataIndex: 'name'
        }, {
            header: "Owner",
            width: 120,
            sortable: true,
            dataIndex: 'owner'
        }, {
            header: "Host",
            width: 120,
            sortable: true,
            dataIndex: 'host'
        }, {
            header: "Port",
            width: 120,
            sortable: true,
            dataIndex: 'port'
        }, {
            header: "Desc",
            width: 135,
            sortable: true,
            dataIndex: 'desc'
        }],
        viewConfig: {
            forceFit: true
        },
        sm: new Ext.grid.RowSelectionModel({
            singleSelect: true
        }),
        width: 600,
        height: 400,
        frame: true,
        tbar: ['Search: ', ' ', new Ext.app.SearchField({
            store: store,
            width: 160
        }), {
            text: 'AddRCA',
            tooltip: 'Add a new RCA',
            //iconCls:'add',
            handler: function(){
                addNewRCA(store);
            }
        }, {
            text: 'DelRCA',
            tooltip: 'Delete a RCA',
            //iconCls:'add',
            handler: function(){
                var _record = grid.getSelectionModel().getSelected();
                if (!_record) {
                    Ext.Msg.alert('Warning!', 'Please select a rca before deleting it');
                    return;
                }
                var rcaname = _record.get('name');
                Ext.Ajax.request({
                    url: '../model/rca_util.jsp?func=del',
                    params: {
                        'rcaname': rcaname
                    },
                    success: function(response){
                        var result = eval("(" + response.responseText + ")");
                        if (result.success) {
                            Ext.Msg.alert('Warning!', 'Successfully delete a RCA ' + rcaname);
                        }
                        else {
                            Ext.Msg.alert('Warning!', 'Failed to delete a RCA ' + rcaname + ':' + result.errors.reason);
                        }
                        //loadFile(responseObject.responseText);
                    },
                    failure: function(responseObject){
                        Ext.Msg.alert('Warning!', 'Failed to delete a rca:' + responseObject);
                    }
                });
            }
        }, {
            text: 'Refresh',
            tooltip: 'Refresh',
            handler: function(){
                store.load();
                //grid.render();
            }
        }]
        //iconCls: 'icon-grid'
    });
    
    var p = new Ext.Panel({
        title: 'Help Hints',
        shadown: true,
        collapsible: true,
        renderTo: 'hint',
        width: 200,
        html: '&nbsp;<p>&nbsp;&nbsp;Register a RCA server by clicking AddRCA button&nbsp;<p>&nbsp;&nbsp;'
    });
    
    var gridEl = Ext.getCmp('rcagrid').el;
    gridEl.mask('Please wait......', 'x-mask-loading');
    
    // trigger the data store load
    store.load({
        callback: function(){
            gridEl.unmask(false);
        }
    });
});
