/*
Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences, Science and Technology Facilities Council, and Xlab d.o.o.
All rights reserved.

This file is part of VOM. VOM is part of XtreemOS, a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details. The XtreemOS project has been developed with the financial support of the European Commission's IST program under contract #FP6-033576.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the XtreemOS project nor the names of its partners and/or contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
Ext.app.SearchField = Ext.extend(Ext.form.TwinTriggerField, {
    initComponent: function(){
        Ext.app.SearchField.superclass.initComponent.call(this);
        this.on('specialkey', function(f, e){
            if (e.getKey() == e.ENTER) {
                this.onTrigger2Click();
            }
        }, this);
    },
    
    validationEvent: false,
    validateOnBlur: false,
    trigger1Class: 'x-form-clear-trigger',
    trigger2Class: 'x-form-search-trigger',
    hideTrigger1: true,
    width: 180,
    hasSearch: false,
    paramName: 'query',
    
    onTrigger1Click: function(){
        if (this.hasSearch) {
            this.el.dom.value = '';
            var o = {
                start: 0
            };
            this.store.baseParams = this.store.baseParams ||
            {};
            this.store.baseParams[this.paramName] = '';
            this.store.reload({
                params: o
            });
            this.triggers[0].hide();
            this.hasSearch = false;
        }
    },
    
    onTrigger2Click: function(){
        var v = this.getRawValue();
        if (v.length < 1) {
            this.onTrigger1Click();
            return;
        }
        var o = {
            start: 0
        };
        this.store.baseParams = this.store.baseParams ||
        {};
        this.store.baseParams[this.paramName] = v;
        this.store.reload({
            params: o
        });
        this.hasSearch = true;
        this.triggers[0].show();
    }
});

Ext.onReady(function(){
    Ext.QuickTips.init();
	// Ext.get('form-ct').setWidth("600px");
	
	var store = new Ext.data.JsonStore({
        url: "../model/list_vo.jsp?all=true",
        totalProperty: "results",
        root: "rows",
        fields: ['id','name','owner','desc','gvid','member']
    });
	
    var grid = new Ext.grid.GridPanel({
    	applyTo: 'form-ct',
        store: store,
        loadMask: true,
		id:'vogrid',
		title: 'Join a VO',
		width: 640,
		height: 400,
		autoScroll:true,
       
        columns: [new Ext.grid.CheckboxSelectionModel(), {
            header: "GVID",
            width: 160,
            sortable: true,
            dataIndex: 'gvid'
        }, {
            header: "VO Name",
            width: 50,
            sortable: true,
            dataIndex: 'name'
        }, {
            header: "VO Owner",
            width: 50,
            sortable: true,
            dataIndex: 'owner'
        },{
            header: "Is Member",
            width: 50,
            sortable: true,
            dataIndex: 'member'
        },{
            header: "Description",
            width: 120,
            sortable: true,
            dataIndex: 'desc'
        }],
        viewConfig: {
            forceFit: true
        },
        sm: new Ext.grid.RowSelectionModel({
            singleSelect: true,
			listeners:{
				rowselect :function(selectionModel){
					if(!selectionModel){
						return ;
					}
					var _record = selectionModel.getSelected();
                	if (!_record) {
                    	return;
                	}
					var leaveavoBtn = Ext.getCmp("leaveavoBtn");
					var joinavoBtn = Ext.getCmp("joinavoBtn");
					var member = _record.get("member");
					if(member=="true"){
						leaveavoBtn.enable();
						joinavoBtn.disable();
					}else{
						leaveavoBtn.disable();
						joinavoBtn.enable();
					}
				}	
			}
        }),
        frame: true,
        tbar: ['Search: ', ' ', new Ext.app.SearchField({
            store: store,
            width: 320
        }),{
            text: 'JoinVO',
			id:'joinavoBtn',
            tooltip: 'Join a new VO',
            handler: function(){
				var _record = grid.getSelectionModel().getSelected();
                if (!_record) {
                    Ext.Msg.alert('Warning!', 'Please select a vo before joining it');
                    return;
                }
				var gvid = _record.get('gvid');
				var voname = _record.get('name');
				Ext.Ajax.request({
                    url: '../model/vo_util.jsp?func=createvoreq',
					params: {'gvid': gvid},
                    success: function(response){
						var result =eval("("+response.responseText+")");
						if (result.success) {
							Ext.Msg.alert('Information', 'Successfully submit your joining request to the VO '+voname);
							store.load();
						}
						else {
							Ext.Msg.alert('Warning!', 'Failed to submit your request to ' + voname + ':' + result.errors.reason);
						}
                    },
                    failure: function(responseObject){
                        Ext.Msg.alert('Warning!', 'Failed to submit your request to  '+voname+':' + responseObject);
                    }
                });
                //addNewVO(store);
            }
        },{
            text: 'LeaveVO',
			id:'leaveavoBtn',
            tooltip: 'Leave a VO',
            handler: function(){
				var _record = grid.getSelectionModel().getSelected();
                if (!_record) {
                    Ext.Msg.alert('Warning!', 'Please select a vo before leaving it');
                    return;
                }
				var gvid = _record.get('gvid');
				var voname = _record.get('name');
				Ext.Ajax.request({
                    url: 'voutil.jsp?func=leavevo',
					params: {'gvid': gvid},
                    success: function(response,option){
						var result =eval("("+response.responseText+")");
						if(result.success)
                        	store.load();
						else{
							Ext.Msg.alert('Warning!', 'Failed to leave VO '+voname+':' + result.errors.reason);
						}
                    },
                    failure: function(responseObject){
                        Ext.Msg.alert('Warning!', 'Failed to leave VO '+voname+':' + responseObject.responseText);
                    }
                });
                //addNewVO(store);
            }
        }, {
            text: 'Refresh',
            tooltip: 'Refresh',
            handler: function(){
                store.load();
            }
        }]
        //iconCls: 'icon-grid'
    });
	
	/*
    myWin = new Ext.Window({
        id: winId,
        width: 560,
        height: 400,
        minWidth: 300,
        minHeight: 200,
        plain: true,
        title: 'Manage Membered VO',
        layout: 'fit',
        border: false,
        closable: true,
        items: grid,
        closeAction: 'hide',
        rendTo: 'form-ct'
    });
    myWin.show();
	*/
	
	grid.getSelectionModel();
	store.load();

	var p = new Ext.Panel({
        title: 'Help Hints',
        shadown: true,
        collapsible:true,
        renderTo: 'hint',
        width:160,
        html: '<p>&nbsp;<p>&nbsp;Choose one VO from the table, then click JoinVO or LeaveVO button<p>&nbsp;<p>Or you could <a href="uni_view.jsp?module=vo&js=vo_owned">create your own VO</a> !'
    });
});

