/*
Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences, Science and Technology Facilities Council, and Xlab d.o.o.
All rights reserved.

This file is part of VOM. VOM is part of XtreemOS, a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details. The XtreemOS project has been developed with the financial support of the European Commission's IST program under contract #FP6-033576.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the XtreemOS project nor the names of its partners and/or contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
Ext.app.SearchField = Ext.extend(Ext.form.TwinTriggerField, {
    initComponent: function(){
        Ext.app.SearchField.superclass.initComponent.call(this);
        this.on('specialkey', function(f, e){
            if (e.getKey() == e.ENTER) {
                this.onTrigger2Click();
            }
        }, this);
    },
    
    validationEvent: false,
    validateOnBlur: false,
    trigger1Class: 'x-form-clear-trigger',
    trigger2Class: 'x-form-search-trigger',
    hideTrigger1: true,
    width: 180,
    hasSearch: false,
    paramName: 'query',
    
    onTrigger1Click: function(){
        if (this.hasSearch) {
            this.el.dom.value = '';
            var o = {
                start: 0
            };
            this.store.baseParams = this.store.baseParams ||
            {};
            this.store.baseParams[this.paramName] = '';
            this.store.reload({
                params: o
            });
            this.triggers[0].hide();
            this.hasSearch = false;
        }
    },
    
    onTrigger2Click: function(){
        var v = this.getRawValue();
        if (v.length < 1) {
            this.onTrigger1Click();
            return;
        }
        var o = {
            start: 0
        };
        this.store.baseParams = this.store.baseParams ||
        {};
        this.store.baseParams[this.paramName] = v;
        this.store.reload({
            params: o
        });
        this.hasSearch = true;
        this.triggers[0].show();
    }
});

Ext.namespace("Ext.ux.grid");  
Ext.ux.grid.SimpleGrid = Ext.extend(Ext.grid.GridPanel, {  
         structure : '',  
         url : '',  
         defaultSortField : '',  
         keyField : '',  
         externalFilters : '',  
         needPage : true,  
         frame : true,  
         animCollapse : true,  
         loadMask : true,  
         viewConfig : {  
             forceFit : true  
         },  
   
         // private  
         initComponent : function() {  
             if (this.structure != '') {  
                 this.initStructure();  
             }  
   
             Ext.ux.grid.SimpleGrid.superclass.initComponent.call(this);  
   
             if (!this.selModel) {  
                 new Ext.grid.RowSelectionModel( {  
                     singleSelect : true		       
                 });  
             }  
         },  
   
	 // private  
         initEvents : function() {  
             Ext.grid.GridPanel.superclass.initEvents.call(this);  
   
             this.getStore().load( {  
                 params : {  
                     start : 0,  
                     limit : 30  
                 }  
             });  
   
             if (this.loadMask) {  
                 this.loadMask = new Ext.LoadMask(this.bwrap, Ext.apply( {  
                     store : this.store  
                 }, this.loadMask));  
             }  
         },  
   
	getSelectedItem: function (actText){
		var _record = this.getSelectionModel().getSelected();
                if (!_record) {
				Ext.MessageBox.show({
					title:'Select a '+actText+' Item',
					msg:'Please select a '+actText+' Item',
					buttons: Ext.MessageBox.OK,  
					icon: Ext.MessageBox.INFO				
				}); 
                    return;
                }

		return _record;
	},
	reload: function(){
			this.show();
			this.refreshGrid();
	 },
	//Refresh
	refreshGrid : function(){
		     this.store.load();
	     },
	addItem: function(){
		if(this.addWin){
			this.addWin.show();
			return;
		}

		var items=[];
		for (var i = 0, len = this.structure.length;i < len; i++) {  
			var c = this.structure[i];  
			if(c.hidden)
				continue;
			var ii = items.length;
			if(c.addItemFunc instanceof Function){
				items[ii]= c.addItemFunc(c.dataIndex);
				items[ii].name = c.dataIndex;
				items[ii].id = c.dataIndex;
				items[ii].isgrid = true;
				continue;
			}
	                items[ii] = {  
                         fieldLabel : c.header,  
                         name : c.dataIndex,  
			 id : c.dataIndex,  
			 value: c.defaultValue? c.defaultValue:'',
                         width : '95%'
                   	};
		       if(c.xtype)
			       items[ii].xtype = c.xtype;
		       if(c.xtype == 'combo'){
				items[ii] = {  
					 xtype:'combo',
	                	         fieldLabel : c.header,  
        	                	 name : c.dataIndex,  
					 id : c.dataIndex+'-combo',  
					 value: c.defaultValue? c.defaultValue:'',
	        	                 width : '95%'				
                	   	};
		       }else if(c.type =='int'){
			       items[ii].allowDecimals=false;
			       items[ii].allowNegative=false;
		               items[ii].blankText='Please input numbers';
		    	       items[ii].emptyTex='Please input numbers';
			       items[ii].xtype = 'numberfield';
		       }
		       if(c.options)
		       		items[ii]=Ext.apply(c.options,items[ii]);	       
		       if(c.addOptions)
		       		items[ii]=Ext.apply(c.addOptions,items[ii]);	       
                 }  
              
		var cancelFn = function (){
			this.addWin.hide();
		};
		cancelFn = cancelFn.createDelegate(this);

		var okFn = function(){
			var form = this.addForm.getForm();
			form.model = this;
			form.submit({
			    waitTitle: 'Creating ......',
			    waitMsg: 'Please wait during the creating process',
			    success: function(form, action){
				form.reset();
				form.model.addWin.hide();
				form.model.refreshGrid();
			    },
			    failure: function(form, action){
				if (action.failureType == 'server') {
				    obj = Ext.util.JSON.decode(action.response.responseText);
					Ext.MessageBox.show({
						title:'Create a '+this.prefix,
						msg:'Server creates the '+this.prefix+' unsuccessfully:'+ obj.errors.reason,
						buttons: Ext.MessageBox.OK,  
						icon: Ext.MessageBox.ERROR				
					}); 
				}else {
					Ext.MessageBox.show({
						title:'Create a '+this.prefix,
						msg:'Server is unreachable:' + action.response.responseText,
						buttons: Ext.MessageBox.OK,  
						icon: Ext.MessageBox.ERROR				
					}); 
                            
                        	}
                        	form.model.addWin.hide();
                    	 }//end failure
                      });
		};
		okFn = okFn.createDelegate(this);
		
		this.addForm = new Ext.FormPanel({
        		width: 400,
		        autoheight: true,
		        minWidth: 300,
		        minHeight: 200,
		        layout: 'form',
		        border: false,
		        closeAction: 'hide',
		        closable: true,
			labelAlign: 'right',
		        frame:true,
			url: this.addInfo.url,
			bodyStyle: 'padding:15px',
			//anchor:'95%',
			defaultType:'textfield',
			items:items,
			buttons:[{
				text:'OK',
				handler:okFn
			},{
				text:'Cancel',
				handler:cancelFn
			}]
			
    		});
		this.addWin =  new Ext.Window({
			width: 400,
			autoheight: true,
			title: 'Create a '+this.prefix,
			border: true,
			closable: true,
			items: this.addForm
    		});
		this.addWin.show(this);
	},
	delItem: function(){
		 var r = this.getSelectedItem('Delete a '+this.prefix);
		 if(!r) return;
		 var rid = r.get(this.keyField);
		 var successFn = function(response,option){
				var result =eval("("+response.responseText+")");
				if(result.success)
                        		this.refreshGrid();
				else{
					Ext.MessageBox.show({
						title:'Delete a '+this.prefix,
						msg:'Sorry, Delete the selected '+this.prefix+' unsuccessfully :' + result.errors.reason,
						buttons: Ext.MessageBox.OK,  
						icon: Ext.MessageBox.ERROR				
					}); 
				}									
                  };
 		 successFn = successFn.createDelegate(this);		 
		 var failureFn =function(responseObject){
				Ext.MessageBox.show({
					title:'Delete a '+this.prefix,
					msg:'Sorry, Delete the selected '+this.prefix+ ' unsuccessfully:' + responseObject.responseText,
					buttons: Ext.MessageBox.OK,  
					icon: Ext.MessageBox.ERROR				
				}); 
                    	    }; 
		 failureFn = failureFn.createDelegate(this);
		 Ext.Ajax.request({
			    url: this.delInfo.url,//'../model/ruleutil.php?func=deliprule',
			    params: {'id': rid},
                    	    success: successFn,
                    	    failure: function(responseObject){
				Ext.MessageBox.show({
					title:'Delete a '+this.prefix,
					msg:'Sorry, Delete the selected '+this.prefix+' unsuccessfully:' + responseObject.responseText,
					buttons: Ext.MessageBox.OK,  
					icon: Ext.MessageBox.ERROR				
				});                         	
                    	    }		
                });
	},
	editItem: function(){
		var record = this.getSelectedItem('Edit a '+this.prefix);
		if(!record) return;

		if(this.editWin){
			this.editWin.show();

			//update data
			var items = this.editForm.findBy(function(){return true;});
			for(var i = 0; i< items.length;i++){
				if(!(items[i].setValue instanceof Function))
					continue;
				var val = record.get(items[i].name);
				items[i].setValue(val);
			}
			return;
		}

		var items=[];
		for (var i = 0, len = this.structure.length;i < len; i++) {  
			var c = this.structure[i];  
			var ii = items.length;
			if(c.hidden){
				items[ii] = {
					xtype:'hidden',
					id: c.dataIndex,
					value: record.get(c.dataIndex)
				};
				continue;
			}
			if(c.editItemFunc instanceof Function){
				items[ii]= c.editItemFunc(c.dataIndex);
				items[ii].name = c.dataIndex;
				items[ii].id = c.dataIndex;
				items[ii].isgrid = 1;
				items[ii].setValue(record.get(c.dataIndex));
				continue;
			}
	                items[ii] = {  
                         fieldLabel : c.header,  
                         name : c.dataIndex,  
			 id : c.dataIndex,  
			 value: record.get(c.dataIndex),
                         width : '95%'
			 //xtype: c.type=='int'?'NumberField':'textField',			 	 
                         // numbers is right-aligned
                         //align : c.type == 'numeric' ? 'right' : 'left',  
                         // Date is in format of xxxx-xx-xx                           
                   	};
		       if(c.xtype)
			       items[ii].xtype = c.xtype;
		       if(c.xtype == 'combo'){
				items[ii] = {  
					 xtype:'combo',
	                	         fieldLabel : c.header,  
        	                	 name : c.dataIndex,  
					 id : c.dataIndex+'-combo',  
					 value: record.get(c.dataIndex),
	        	                 width : '95%'				
                	   	};
		       }else if(c.type =='int'){
			       items[ii].allowDecimals=false;
			       items[ii].allowNegative=false;
		               items[ii].blankText='Please Input numbers';
		    	       items[ii].emptyTex='Please Input numbers';
			       items[ii].xtype = 'numberfield';
		       }
		       if(c.options)
		       		items[ii]=Ext.apply(c.options,items[ii]);
		       if(c.editOptions)
		       		items[ii]=Ext.apply(c.editOptions,items[ii]);	       
                 }  
              
		var cancelFn = function (){
			this.editWin.hide();
		};
		cancelFn = cancelFn.createDelegate(this);

		var okFn = function(){
			var form = this.editForm.getForm();
			form.model = this;
			form.submit({
			    waitTitle: 'Updating ......',
			    waitMsg: 'Please wait during the updating process',
			    success: function(form, action){
				form.reset();
				form.model.editWin.hide();
				form.model.refreshGrid();
			    },
			    failure: function(form, action){
				if (action.failureType == 'server') {
				    obj = Ext.util.JSON.decode(action.response.responseText);
					Ext.MessageBox.show({
						title:'Edit a '+this.prefix,
						msg:'Server updates '+this.prefix+' unsuccessfully:'+ obj.errors.reason,
						buttons: Ext.MessageBox.OK,  
						icon: Ext.MessageBox.ERROR				
					}); 
				}else {
					Ext.MessageBox.show({
						title:'Edit a '+this.prefix,
						msg:'Server is unreachable:' + action.response.responseText,
						buttons: Ext.MessageBox.OK,  
						icon: Ext.MessageBox.ERROR				
					}); 
                            
                        	}
                        	form.model.editWin.hide();
                    	 }//end failure
                      });
		};
		okFn = okFn.createDelegate(this);
		
		this.editForm = new Ext.FormPanel({
        		width: 400,
		        autoheight: true,
		        minWidth: 300,
		        minHeight: 200,
		        layout: 'form',
		        border: false,
		        closeAction: 'hide',
		        closable: true,
			labelAlign: 'right',
		        frame:true,
			url: this.editInfo.url,
			bodyStyle: 'padding:15px',
			//anchor:'95%',
			defaultType:'textfield',
			items:items,
			buttons:[{
				text:'OK',
				handler:okFn
			},{
				text:'Cancel',
				handler:cancelFn
			}]
			
    		});
		this.editWin =  new Ext.Window({
			width: 400,
			autoheight: true,
			title: 'Edit a '+this.prefix,
			border: true,
			closable: true,
			items: this.editForm
    		});
		this.editWin.show(this);
	},

	selectRow: function(row){
		var sm = this.getSelectionModel();
		if(sm) sm.selectRow(row);
	},
	 // private  
         initStructure : function() { 
	     var globalMe = this; 
             // Do we have other external filters 
             var oFilters = this.externalFilters == '' ? {  
                 filters : []  
             } : {  
                 filters : [],  
                 externalFilters : this.externalFilters  
             };  
             var oCM = new Array();  
             var oRecord = new Array(); 
	     if(this.addInfo || this.delInfo || this.editInfo){
		    oCM[oCM.length] =new Ext.grid.CheckboxSelectionModel();		      
	     }

	     this.sm =new Ext.grid.RowSelectionModel( {  
                     		singleSelect : true,
	 			listeners:{
					rowselect:function(selectionModel){
							var thisGrid = selectionModel.model;
							var r = selectionModel.getSelected();
							if(thisGrid.selectFn)
								thisGrid.selectFn(r,thisGrid);
						}
				}			
                 	});
	     this.sm.model = this;

             // Grid comprises of three components: filters,column and record，all of which can be created based on structure
             for (var i = 0, len = this.structure.length;i < len; i++) {  
                 var c = this.structure[i];  
                 //If the field is hidden, no filters and columnMode are created 
                 if (c.hidden == undefined || !c.hidden) {  
                     oFilters.filters[oFilters.filters.length] = {  
                         // default type string  
                         type : c.type == undefined ? 'string' : c.type,  
                         dataIndex : c.dataIndex  
                     };  
                     oCM[oCM.length] = {  
                         header : c.header,  
                         dataIndex : c.dataIndex,  
                         width : c.width,  
                         //type numberic is right-aligned
                         align : c.type == 'numeric' ? 'right' : 'left',  
                         //type date is in format of xxxx-xx-xx  
                         renderer : c.type == 'date'  
                                 ? Ext.util.Format.dateRenderer('Y-m-d')  
                                 : (c.renderer?c.renderer:Ext.grid.ColumnModel.defaultRenderer)  
                   };  
                 }  
                 oRecord[oRecord.length] = {  
                     name : c.dataIndex,  
                     //default type is string
                     type : c.type? c.type:'string'// == 'date' ? 'date' : 'string'  
                 };  
             }  
             //create columnModel  
             this.cm = new Ext.grid.ColumnModel(oCM);  
             this.colModel = this.cm;  
             //default is sortable  
             this.cm.defaultSortable = true;  
   
             //var filters = new Ext.ux.grid.GridFilters(oFilters);  
             // create record
             var record = Ext.data.Record.create(oRecord);  
             var reader = new Ext.data.JsonReader( {  
                 totalProperty : "results",  
                 root : "rows",  
                 id : this.keyField  
             }, record);  
             this.ds = new Ext.data.Store( {  
                 proxy : new Ext.data.HttpProxy( {  
                     url : this.url  
                 }),  
                 reader : reader,  
                 sortInfo : {  
                     field : this.defaultSortField,  
                     direction : 'ASC'  
                 }  
                 //remoteSort : true  
             });  
             this.store = this.ds;  
             //create page toolbar  
             if (this.needPage) {  
                 var pagingToolbar = new Ext.PagingToolbar( {  
                     displayInfo : true,  
                     displayMsg : 'Current records: {0} - {1} Total: {2}',  
                     emptyMsg : 'No records returned',  
                     store : this.ds  
                     //plugins : [/*filters,*/ new Ext.ux.PageSizePlugin()]  
                 });  
                 pagingToolbar.pageSize = 30;  
                 this.bbar = pagingToolbar;  
                 this.bottomToolbar = this.bbar;  
             }
	     //TopToolbar
	     var tbar=[];
	     if(this.searchInfo){
		tbar[tbar.length] ='Search:';
	       	tbar[tbar.length]=' ';
		tbar[tbar.length] =new Ext.app.SearchField({store: this.store,width: 320});
		tbar[tbar.length]={xtype:'tbfill'};
	     }

	     if(this.addInfo){
		tbar[tbar.length] ={text: 'Create',handler:function(){globalMe.addItem();}};
		tbar[tbar.length] =' ';
	     }

	     if(this.editInfo){
		tbar[tbar.length] ={text:'Edit',handler:function(){globalMe.editItem();}};
		tbar[tbar.length] =' ';
	     }

	     if(this.delInfo){
		tbar[tbar.length] ={text:'Delete',handler:function(){globalMe.delItem();}};
		tbar[tbar.length] =' ';
	     }
	     //if(this.refresh){
		tbar[tbar.length] ={text:'Refresh',handler: function(){globalMe.refreshGrid();}};
		
	     //}

	     if(tbar.length >0)
		     this.tbar= new Ext.Toolbar({items:tbar});
             // add filter to grid 
             //this.plugins = filters;  
	     this.store.on('load',function(){
			     //grid.getSelectionModel().selectFirstFow();
			     if(globalMe.store.getCount()>0)
				     globalMe.selectRow(0);
			    });
         }  
     });  
Ext.reg('simpleGrid', Ext.ux.grid.SimpleGrid); 


Ext.namespace("Ext.ux.form");
Ext.ux.form.SimpleForm= Ext.extend(Ext.FormPanel, {
        structure : '',  
	autoheight: true,
	layout: 'form',
	border: true,
	closeAction: 'hide',
	closable: false,
	labelAlign: 'right',
	frame:true,
	bodyStyle: 'padding:15px',
	//anchor:'95%',
	defaultType:'textfield',
	buttons:[{
		text:'OK',
		handler:this.okFn
	},{
		text:'Reset',
		handler:this.cancelFn
	}],  

	okFn: function (){
		var form = this.getForm();
		form.model = this;
		form.submit({
		    success: function(form, action){
			    Ext.MessageBox.show({
				title:form.model.title,
				msg:'Congratulations, Server updates '+form.model.prefix+' successfully',
				buttons: Ext.MessageBox.OK,  
				icon: Ext.MessageBox.INFO				
				});
			     form.reset();			
		    },
		    failure: function(form, action){
			if (action.failureType == 'server') {
			    obj = Ext.util.JSON.decode(action.response.responseText);
				Ext.MessageBox.show({
					title:form.model.title,
					msg:'Server failed to update '+form.model.prefix+' :'+ obj.errors.reason,
					buttons: Ext.MessageBox.OK,  
					icon: Ext.MessageBox.ERROR				
				}); 
			}else {
				Ext.MessageBox.show({
					title:form.model.title,
					msg:'Server is unreachable!', //+ action.response.responseText,
					buttons: Ext.MessageBox.OK,  
					icon: Ext.MessageBox.ERROR				
				}); 
		    
			}
		 }//end failure
	      });
	},

	resetFn: function (){
		var form = this.getForm();
		form.reset();
	}, 
        viewConfig : {  
             forceFit : true  
        },
	listeners:{
	     render: function(){
		var r = this.resetFn.createDelegate(this);	
		window.setTimeout(r,1);
   	     }			     
	},
	// private  
        initComponent : function() {  
             if (this.structure != '') {  
                 this.initStructure();  
             }  
   
             Ext.ux.form.SimpleForm.superclass.initComponent.call(this);  
         },  
   
	 // private  
         initEvents : function() {  
             Ext.FormPanel.superclass.initEvents.call(this);                 
         }, 
        initStructure : function() {
		var structFn= function(c){
			if(c.hidden)
				return false;
			var item;
			if(c.addItemFunc instanceof Function){
				item= c.addItemFunc(c.dataIndex);
				item.name = c.dataIndex;
				item.id = c.dataIndex;
				item.isgrid = true;
				return item;
			}
	                item = {  
                         fieldLabel : c.header,  
                         //name : c.dataIndex,  
			 id : c.dataIndex,  
			 value: c.defaultValue? c.defaultValue:'',
                         width : '85%'
			 //xtype: c.type=='int'?'NumberField':'textField',			 	 
                         // 类型为数字则右对齐  
                         //align : c.type == 'numeric' ? 'right' : 'left',  
                         // 类型为日期则生成格式为xxxx-xx-xx                           
                   	};
		       if(c.xtype)
			       item.xtype = c.xtype;
		       if(c.xtype == 'combo'){
				item ={ 
					xtype:'combo',
					fieldLabel : c.header, 
					selectOnFocus : true,
					triggerAction : "all" 
				};
				if(c.defaultValue)
					item.value = c.defaultValue;
				if(c.store)
					item.store = c.store;
				if(c.displayField)
					item.displayField = c.displayField;
				if(c.valueField)
					item.valueField = c.valueField;
				if(c.hiddenName)
					item.hiddenName = c.hiddenName;
				//return item;			       
		       }else if(c.xtype =='fieldset'){
			       item ={
					xtype:'fieldset',
					title: c.title
			       };
			       if(c.options){
				   var its = c.options.items;
			       	   c.options.items = null;
				   item=Ext.apply(c.options,item);
				   if(its){
					var itss =[];
				   	for(var i =0, len = its.length;i <len; i++){
					   itss[itss.length]=structFn(its[i]);
				   	}
					item.items = itss;
				   }
				}
		       }else if(c.type =='int'){
			       item.allowDecimals=false;
			       item.allowNegative=false;
		               item.blankText='Please Input numbers';
		    	       item.emptyTex='Please input numbers';
			       item.xtype = 'numberfield';
		       }
		       if(c.vtype)
			       item.vtype =c.vtype;
		       if(c.options)
		       		item=Ext.apply(c.options,item);	       
		       if(c.addOptions)
		       		item=Ext.apply(c.addOptions,item);
		       return item;
		};
		var items=[];
		for (var i = 0, len = this.structure.length;i < len; i++) {  
			var c = this.structure[i];  
			var item = structFn(c);
			if(item!=false)
				items[items.length] = item;
		}
		this.items = items;
		//okFn = this.okFn.createDelegate(this);		
		//resetFn = this.resetFn.createDelegate(this);
		this.buttons = [{
				text:'OK',
				handler:this.okFn.createDelegate(this)
			},{
				text:'Reset',
				handler:this.resetFn.createDelegate(this)
			}];
	}
});	
Ext.reg('simpleForm', Ext.ux.form.SimpleForm);
