/*
Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences, Science and Technology Facilities Council, and Xlab d.o.o.
All rights reserved.

This file is part of VOM. VOM is part of XtreemOS, a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details. The XtreemOS project has been developed with the financial support of the European Commission's IST program under contract #FP6-033576.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the XtreemOS project nor the names of its partners and/or contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
Ext.app.SearchField = Ext.extend(Ext.form.TwinTriggerField, {
    initComponent: function(){
        Ext.app.SearchField.superclass.initComponent.call(this);
        this.on('specialkey', function(f, e){
            if (e.getKey() == e.ENTER) {
                this.onTrigger2Click();
            }
        }, this);
    },
    
    validationEvent: false,
    validateOnBlur: false,
    trigger1Class: 'x-form-clear-trigger',
    trigger2Class: 'x-form-search-trigger',
    hideTrigger1: true,
    width: 180,
    hasSearch: false,
    paramName: 'query',
    
    onTrigger1Click: function(){
        if (this.hasSearch) {
            this.el.dom.value = '';
            var o = {
                start: 0
            };
            this.store.baseParams = this.store.baseParams ||
            {};
            this.store.baseParams[this.paramName] = '';
            this.store.reload({
                params: o
            });
            this.triggers[0].hide();
            this.hasSearch = false;
        }
    },
    
    onTrigger2Click: function(){
        var v = this.getRawValue();
        if (v.length < 1) {
            this.onTrigger1Click();
            return;
        }
        var o = {
            start: 0
        };
        this.store.baseParams = this.store.baseParams ||
        {};
        this.store.baseParams[this.paramName] = v;
        this.store.reload({
            params: o
        });
        this.hasSearch = true;
        this.triggers[0].show();
    }
});

var createAVO = function(store){
    var winId = "createAVOId";
    
    var myWin = Ext.getCmp(winId);
    if (myWin != null) {
        myWin.show();
        return;
    }
    
    function checkSubmited(){
        var name = Ext.getCmp('voname');
        if (!name.el.dom.value == null || name.el.dom.value.length <= 0) {
            Ext.MessageBox.alert("Create A VO", "vo name is null");
            name.focus();
            return false;
        }
        
        return true;
    }
    
    var fs = new Ext.FormPanel({
	width: 640,
        labelWidth: 70,
        bodyStyle: 'padding:15px',
        border: false,
        frame: true,
        waitMsgTarget: true,
         monitorValid:true,
        url: '../model/vo_owned.jsp',
        items: [{
            layout: 'column',
            items: [{
                columnWidth: .7,
                layout: 'form',
                items: [{
                    xtype: 'textfield',
                    fieldLabel: 'VO Name',
                    name: 'voname',
                    id: 'voname',
                    anchor: '95%',
                    allowBlank: false
                }, {
                
                    xtype: "checkbox",
                    fieldLabel: "Options",
                    boxLabel: "Automatic approving of requests(disabled)",
                    name: "autoapprove",
                    id: "autoapprove",
                    inputValue: "cbvalue",
                	disabled: true
                }]
            }]
        }, {
            xtype: 'htmleditor',
            id: 'desc',
            fieldLabel: 'VO Description',
            height: 120,
            anchor: '98%'
        }],
        
        buttons: [{
            text: 'Create',
            handler: function(){
                if (!checkSubmited()) 
                    return false;
                var form = fs.getForm();
                var node_id = Ext.getCmp('autoapprove');
                node_id.el.dom.value = node_id.checked;
                var voname = Ext.getCmp('voname').el.dom.value;
                
                form.submit({
                    waitTitle: 'Connecting',
                    waitMsg: 'Waiting...',
                    success: function(form, action){
                        Ext.Msg.alert('Create a VO', "Successfully create a vo:!" + voname);
                        form.reset();
                        myWin.close();
                        if (store != null) {
                            store.load();
                        }
                    },
                    failure: function(form, action){
                        if (action.failureType == 'server') {
                            obj = Ext.util.JSON.decode(action.response.responseText);
                            Ext.Msg.alert('Server Failed!', obj.errors.reason);
                        }
                        else {
                            Ext.Msg.alert('Warning!', 'Server is unreachable:' + action.response.responseText);
                        }
                        myWin.close();
                    }
                });
            }
        }, {
            text: 'Cancel',
            handler: function(){
                myWin.close();
            }
        }]
    });
    
    myWin = new Ext.Window({
        id: winId,
        width: 640,
        height: 460,
        minWidth: 300,
        minHeight: 200,
        plain: true,
        title: 'Create A New VO',
        layout: 'fit',
        border: false,
        closeAction: 'hide',
        closable: true,
        items: fs,
        rendTo: 'form-ct'
    });
    myWin.show(this);
    
    //fs.render('form-ct');  
};

Ext.onReady(function(){
    Ext.QuickTips.init();
    
    var vostore = new Ext.data.JsonStore({
        url: "../model/list_vo.jsp",
        totalProperty: "results",
        root: "rows",
        id: "id",
        fields: ['id', 'name', 'desc', 'gvid', 'member', 'owner']
    });
    var vogrid = new Ext.grid.GridPanel({
        store: vostore,
        loadMask: true,
        id: 'voownedgrid',
        applyTo: 'form-ct',
		layout:'fit',
		title:'Manage Owned VOs',
        columns: [new Ext.grid.CheckboxSelectionModel(), {
            header: "Id",
            width: 30,
            sortable: true,
            dataIndex: 'id'
        }, {
            header: "Name",
            width: 80,
            sortable: true,
            dataIndex: 'name'
        }, {
            header: "Is Memember",
            width: 80,
            sortable: true,
            dataIndex: 'member'
        }, {
            header: "Owner",
            width: 80,
            sortable: true,
            dataIndex: 'owner'
        }, {
            header: "Desc",
            width: 100,
            sortable: true,
            dataIndex: 'desc'
        }],
        viewConfig: {
            forceFit: true
        },
        sm: new Ext.grid.RowSelectionModel({
            singleSelect: true
        }),
        width: 600,
        height: 400,
        frame: true,
        tbar: ['Search: ', ' ', new Ext.app.SearchField({
            store: vostore,
            width: 320
        }), {
            text: 'CreateVO',
            tooltip: 'Create a New VO',
            //iconCls:'add'
            handler: function(){
                createAVO(vostore);
            }
        }, {
            text: 'DeleteVO',
            tooltip: 'Del a VO',
            //iconCls:'add',
            handler: function(){
                var _vorecord = vogrid.getSelectionModel().getSelected();
                if (!_vorecord) {
                    Ext.Msg.alert('Warning!', 'Please select a VO');
                    return;
                }
                
                var gvid = _vorecord.get("gvid");
                var voname = _vorecord.get("name");
		var body = Ext.getCmp('voownedgrid').el;
                //var body = vogrid.el;
                body.mask('Please wait......', 'x-mask-loading');
                
                Ext.Ajax.request({
                    url: '../model/vo_util.jsp?func=deletevo',
                    params: {
                        'gvid': gvid
                    },
                    success: function(response){
                        body.unmask(false);
		   	var res_str= response.responseText.replace(/(^\s*)|(\s*$)|(\n)|(\r)/g, ""); 
			var result = eval("("+res_str+")");
                        //var result = eval("(" + response.responseText + ")");
                        if (result.success) {
                            Ext.Msg.alert('Congratulations!', 'Successfully delete a VO ' + voname);
                        }
                        else {
                            Ext.Msg.alert('Sorry!', 'Failed to delete a VO ' + voname + ':' + result.errors.reason);
                        }
			vostore.load();
                    },
                    failure: function(response){
                        body.unmask(false);
                        Ext.Msg.alert('Sorry!', 'Failed to delete a VO:' + response.responseText);
			vostore.load();
                    }
                });
            }
        }, {
            text: 'Refresh',
            tooltip: 'Refresh',
            handler: function(){
                vostore.load();
                //grid.render();
            }
        }]
    });
    
	var p = new Ext.Panel({
        title: 'Help Hints',
        shadown: true,
        collapsible: true,
        renderTo: 'hint',
        width: 200,
        html: '&nbsp;<p>&nbsp;&nbsp;Click CreateVO button on the toolbar to create one new VO of your own. &nbsp;<p>&nbsp;&nbsp;'
    });
	
    var gridEl = Ext.getCmp('voownedgrid').el;
    gridEl.mask('Please wait......', 'x-mask-loading');
    
    // trigger the data store load
    vostore.load({
        callback: function(){
            gridEl.unmask(false);
        }
    });
});
