/*
Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences, Science and Technology Facilities Council, and Xlab d.o.o.
All rights reserved.

This file is part of VOM. VOM is part of XtreemOS, a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details. The XtreemOS project has been developed with the financial support of the European Commission's IST program under contract #FP6-033576.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the XtreemOS project nor the names of its partners and/or contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
Ext.app.SearchField = Ext.extend(Ext.form.TwinTriggerField, {
    initComponent: function(){
        Ext.app.SearchField.superclass.initComponent.call(this);
        this.on('specialkey', function(f, e){
            if (e.getKey() == e.ENTER) {
                this.onTrigger2Click();
            }
        }, this);
    },
    
    validationEvent: false,
    validateOnBlur: false,
    trigger1Class: 'x-form-clear-trigger',
    trigger2Class: 'x-form-search-trigger',
    hideTrigger1: true,
    width: 180,
    hasSearch: false,
    paramName: 'query',
    
    onTrigger1Click: function(){
        if (this.hasSearch) {
            this.el.dom.value = '';
            var o = {
                start: 0
            };
            this.store.baseParams = this.store.baseParams ||
            {};
            this.store.baseParams[this.paramName] = '';
            this.store.reload({
                params: o
            });
            this.triggers[0].hide();
            this.hasSearch = false;
        }
    },
    
    onTrigger2Click: function(){
        var v = this.getRawValue();
        if (v.length < 1) {
            this.onTrigger1Click();
            return;
        }
        var o = {
            start: 0
        };
        this.store.baseParams = this.store.baseParams ||
        {};
        this.store.baseParams[this.paramName] = v;
        this.store.reload({
            params: o
        });
        this.hasSearch = true;
        this.triggers[0].show();
    }
});

Ext.onReady(function(){
    Ext.QuickTips.init();
    
    var store = new Ext.data.JsonStore({
        url: "../model/req_util.jsp?type=voreq&func=list",
        totalProperty: "results",
        root: "rows",
        id: "reqid",
        fields: ['reqid', 'username', 'voowner', 'gvid', 'guid','voname']
    });
    var grid = new Ext.grid.GridPanel({
        store: store,
		id: 'voownedgrid',
        //loadMask: true,
        id: 'voreqgrid',
		applyTo: 'form-ct',
		layout:'fit',
		title:'Approve VO Requests',
        columns: [new Ext.grid.CheckboxSelectionModel(), {
            header: "Id",
            width: 30,
            sortable: true,
            dataIndex: 'reqid'
        }, {
            header: "VO",
            width: 60,
            sortable: true,
            dataIndex: 'voname'
        }, {
            header: "VO Owner",
            width: 60,
            sortable: true,
            dataIndex: 'voowner'
        }, {
            header: "Applicant",
            width: 200,
            sortable: true,
            dataIndex: 'username'
        }],
        viewConfig: {
            forceFit: true
        },
        sm: new Ext.grid.RowSelectionModel({
            singleSelect: true
        }),
        width: 600,
        height: 400,
        frame: true,
        tbar: ['Search: ', ' ', new Ext.app.SearchField({
            store: store,
            width: 320
        }), {
            text: 'Approve',
            tooltip: 'Approve the Request-Add the resource to VO',
            handler: function(){
                var _record = grid.getSelectionModel().getSelected();
                if (!_record) {
                    Ext.Msg.alert('Warning!', 'Please select a requset!');
                    return;
                }
                var reqid = _record.get("reqid");
                var body = grid.el;
                body.mask('Please wait......', 'x-mask-loading');
                
                Ext.Ajax.request({
                    url: '../model/req_util.jsp?func=approve&type=voreq',
                    params: {
                        'reqid': reqid
                    },
                    success: function(response){
                        body.unmask(false);
                        var result = eval('(' + response.responseText + ')');
                        if (result.success) {
                            Ext.Msg.alert('Congratulations!', 'Successfully Approve a vo request ');
                        }
                        else {
                            Ext.Msg.alert('Warning!', 'Failed to Approve a vo request:' + result.errors.reason);
                        }
                        store.load();
                    },
                    failure: function(responseObject){
                        Ext.Msg.alert('Warning!', 'Approve a vo request:' + responseObject);
                    }
                });
            }
        }, {
            text: 'Decline',
            tooltip: 'Decline the Request',
            //iconCls:'add',
            handler: function(){
                var _record = grid.getSelectionModel().getSelected();
                if (!_record) {
                    Ext.Msg.alert('Warning!', 'Please select a VO request');
                    return;
                }
                var reqid = _record.get("reqid");
                var body = grid.el;
                body.mask('Please wait......', 'x-mask-loading');
                
                Ext.Ajax.request({
                    url: '../model/req_util.jsp?func=decline&type=voreq',
                    params: {
                        'reqid': reqid
                    },
                    success: function(response){
                        body.unmask(false);
                        var result = eval("(" + response.responseText + ")");
                        if (result.success) {
                            Ext.Msg.alert('Warning!', 'Successfully Decline a VO request');
                        }
                        else {
                            Ext.Msg.alert('Warning!', 'Failed to Decline a VO request:' + result.errors.reason);
                        }
                        store.load();
                    },
                    failure: function(responseObject){
                        Ext.Msg.alert('Warning!', 'Failed to Decline a VO request:' + responseObject);
                    }
                });
            }
        }, {
            text: 'Refresh',
            tooltip: 'Refresh',
            handler: function(){
                store.load();
            }
        }]
        //iconCls: 'icon-grid'
    });
    
	var p = new Ext.Panel({
        title: 'Help Hints',
        shadown: true,
        collapsible: true,
        renderTo: 'hint',
        width: 200,
        html: '&nbsp;<p>&nbsp;&nbsp;Warning: Manage the requests that want to join your own VOs &nbsp;<p>&nbsp;&nbsp;'
    });
	
	grid.render();
    var gridEl = Ext.getCmp('voreqgrid').el;
    gridEl.mask('Please wait......', 'x-mask-loading');
    
    // trigger the data store load
    store.load({
        callback: function(){
            gridEl.unmask(false);
        }
    });
});
